/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.ParametrizedCommandTest.java
 * Created on: Nov 18, 2016
 */
package fr.bigeon.gclc.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;

import org.junit.Test;

import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.InvalidParameterException;
import fr.bigeon.gclc.manager.PipedConsoleManager;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class ParametrizedCommandTest {

    /** Test method for
     * {@link fr.bigeon.gclc.command.ParametrizedCommand#addParameter(java.lang.String, boolean, boolean)}. */
    @Test
    public final void testAddParameter() {
        ParametrizedCommand cmd = new ParametrizedCommand(null, "name") {

            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                return null;
            }
        };
        cmd = new ParametrizedCommand(null, "name", true) {

            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                return null;
            }
        };
        // XXX Boolean flag should not be specified mandatory! They are by
        // nature qualified
        final String str = "str";
        try {
            assertTrue(cmd.getBooleanParameters().isEmpty());
            assertTrue(cmd.getStringParameters().isEmpty());
            cmd.addBooleanParameter("boolFlag");
            assertEquals(1, cmd.getBooleanParameters().size());
            assertTrue(cmd.getStringParameters().isEmpty());
            cmd.addStringParameter(str, false);
            assertEquals(1, cmd.getBooleanParameters().size());
            assertEquals(1, cmd.getStringParameters().size());
            assertFalse(cmd.isNeeded(str));
            cmd.addBooleanParameter("boolFlag");
            assertEquals(1, cmd.getBooleanParameters().size());
            assertEquals(1, cmd.getStringParameters().size());
            cmd.addStringParameter(str, true);
            assertEquals(1, cmd.getBooleanParameters().size());
            assertEquals(1, cmd.getStringParameters().size());
            assertTrue(cmd.isNeeded(str));
            cmd.addStringParameter(str, false);
            assertEquals(1, cmd.getBooleanParameters().size());
            assertEquals(1, cmd.getStringParameters().size());
            assertTrue(cmd.isNeeded(str));
        } catch (final InvalidParameterException e) {
            fail("Unexpected error in addition of legitimate parameter");
            assertNotNull(e);
        }
    }

    /** Test method for
     * {@link fr.bigeon.gclc.command.ParametrizedCommand#execute(java.lang.String[])}. */
    @Test
    public final void testExecute() {
        final String addParam = "additional";
        final String str1 = "str1";
        final String str2 = "str2";
        final String bool1 = "bool1";
        final String bool2 = "bool2";

        // Test on command with no needed elements
        ParametrizedCommand cmd = new ParametrizedCommand("name", false) {
            private boolean evenCall = true;

            @Override
            protected void doExecute(final CommandParameters parameters) {
                assertTrue(parameters.getBooleanArgumentKeys().isEmpty());
                assertTrue(parameters.getStringArgumentKeys().isEmpty());
                if (evenCall) {
                    assertTrue(parameters.getAdditionals().isEmpty());
                    evenCall = false;
                } else {
                    assertEquals(1, parameters.getAdditionals().size());
                    assertTrue(parameters.getAdditionals().contains(addParam));
                    evenCall = true;
                }
            }

            @Override
            public String tip() {
                return "";
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        try {
            cmd.execute();
            cmd.execute("-" + addParam);
            cmd.execute(addParam);
            cmd.execute("-" + addParam, addParam);
        } catch (final CommandRunException e) {
            assertNull(e);
            fail("unepected error");
        }
        cmd = new ParametrizedCommand("name", false) {
            private int call = 0;
            {

                try {
                    addStringParameter(str1, false);
                    addStringParameter(str2, false);
                    addBooleanParameter(bool1);
                    addBooleanParameter(bool2);
                } catch (final InvalidParameterException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }

            @Override
            protected void doExecute(final CommandParameters parameters) {
                assertEquals(2, parameters.getBooleanArgumentKeys().size());
                assertEquals(2, parameters.getStringArgumentKeys().size());
                switch (call) {
                case 0:
                case 1:
                case 2:
                case 3:
                    assertNull(parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertFalse(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call++;
                    break;
                case 4:
                    assertEquals(str2, parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertFalse(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call++;
                    break;
                case 5:
                    assertEquals(str2, parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertTrue(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call = 0;
                    break;
                default:
                    break;
                }
            }

            @Override
            public String tip() {
                return "";
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        try {
            cmd.execute();
            cmd.execute("-" + addParam);
            cmd.execute(addParam);
            cmd.execute("-" + addParam, addParam);
            cmd.execute("-" + str1, str2);
            cmd.execute("-" + str1, str2, "-" + bool1);
        } catch (final CommandRunException e) {
            assertNull(e);
            fail("unepected error");
        }
        cmd = new ParametrizedCommand("name", true) {
            private int call = 0;
            {

                try {
                    addStringParameter(str1, false);
                    addStringParameter(str2, false);
                    addBooleanParameter(bool1);
                    addBooleanParameter(bool2);
                } catch (final InvalidParameterException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
            @Override
            protected void doExecute(final CommandParameters parameters) {
                assertEquals(2, parameters.getBooleanArgumentKeys().size());
                assertEquals(2, parameters.getStringArgumentKeys().size());
                switch (call) {
                case 0:
                    assertNull(parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertFalse(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call++;
                    break;
                case 1:
                    assertEquals(str2, parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertFalse(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call++;
                    break;
                case 2:
                    assertEquals(str2, parameters.get(str1));
                    assertNull(parameters.get(str2));
                    assertTrue(parameters.getBool(bool1));
                    assertFalse(parameters.getBool(bool2));
                    call = 0;
                    break;
                default:
                    break;
                }
            }

            @Override
            public String tip() {
                return "";
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        try {
            cmd.execute();
            cmd.execute("-" + str1, str2);
            cmd.execute("-" + str1, str2, "-" + bool1);
        } catch (final CommandRunException e) {
            assertNull(e);
            fail("unexpected error");
        }
        try {
            cmd.execute(addParam);
            fail("Strict should fail with unexpected argument");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute("-" + addParam);
            fail("Strict should fail with unexpected argument");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        // Test on command with missing needed elements
        cmd = new ParametrizedCommand("name", false) {
            private final int call = 0;
            {

                try {
                    addStringParameter(str1, true);
                    addStringParameter(str2, false);
                    addBooleanParameter(bool1);
                    addBooleanParameter(bool2);
                } catch (final InvalidParameterException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
            @Override
            protected void doExecute(final CommandParameters parameters) {
                assertEquals(str2, parameters.get(str1));
            }

            @Override
            public String tip() {
                return "";
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        try {
            cmd.execute("-" + str1, str2);
            cmd.execute("-" + str1, str2, "-" + bool1);
            cmd.execute("-" + str1, str2, "-" + addParam);
            cmd.execute("-" + str1, str2, addParam);
            cmd.execute("-" + str1, str2, "-" + addParam, addParam);
        } catch (final CommandRunException e) {
            assertNull(e);
            fail("unepected error");
        }
        try {
            cmd.execute();
            fail("needed " + str1 + " not provided shall fail");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        cmd = new ParametrizedCommand("name", true) {
            private final int call = 0;
            {

                try {
                    addStringParameter(str1, true);
                    addStringParameter(str2, false);
                    addBooleanParameter(bool1);
                    addBooleanParameter(bool2);
                } catch (final InvalidParameterException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }

            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
                assertEquals(str2, parameters.get(str1));
            }

            @Override
            public String tip() {
                return "";
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        try {
            cmd.execute("-" + str1, str2);
            cmd.execute("-" + str1, str2, "-" + bool1);
        } catch (final CommandRunException e) {
            assertNull(e);
            fail("unepected error");
        }
        try {
            cmd.execute("-" + str1, str2, addParam);
            fail("Additional parameter should cause failure");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute();
            fail("needed " + str1 + " not provided shall fail");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute("-" + str1, str2, "-" + addParam);
            fail("unepected error");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        try {
            cmd.execute("-" + str1, str2, "-" + addParam, addParam);
            fail("unepected error");
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }
        // TODO Test of interactive not providing and providing all needed
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            cmd = new ParametrizedCommand(test, "name", false) {
                {
                    try {
                        addStringParameter(str1, true);
                        addStringParameter(str2, false);
                        addBooleanParameter(bool1);
                        addBooleanParameter(bool2);
                    } catch (final InvalidParameterException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }
                @Override
                protected void doExecute(final CommandParameters parameters) {
                    assertEquals(str2, parameters.get(str1));
                }

                @Override
                public String tip() {
                    return "";
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            };
            try {
                cmd.execute("-" + str1, str2);
                cmd.execute("-" + str1, str2, "-" + bool1);
                cmd.execute("-" + str1, str2, addParam);
                cmd.execute("-" + str1, str2, "-" + addParam);
                cmd.execute("-" + str1, str2, "-" + addParam, addParam);
            } catch (final CommandRunException e) {
                assertNull(e);
                fail("unepected error");
            }
            try {

                Thread th = new Thread(new Runnable() {

                    @Override
                    public void run() {
                        try {
                            assertEquals("value of " + str1 + "? ",
                                    test.readNextLine());
                            test.type("");
                            assertEquals(
                                    "value of " + str1 + "? (cannot be empty) ",
                                    test.readNextLine());
                            test.type("");
                            assertEquals(
                                    "value of " + str1 + "? (cannot be empty) ",
                                    test.readNextLine());
                            test.type(str2);
                        } catch (final IOException e) {
                            assertNull(e);
                        }
                    }
                });
                th.start();

                cmd.execute();

                th.join();

                th = new Thread(new Runnable() {

                    @Override
                    public void run() {
                        try {
                            assertEquals("value of " + str1 + "? ",
                                    test.readNextLine());
                            test.type(str2);
                        } catch (final IOException e) {
                            assertNull(e);
                        }
                    }
                });
                th.start();

                cmd.execute("-" + addParam);

                th.join();
            } catch (CommandRunException | InterruptedException e) {
                assertNull(e);
                fail("unepected error");
            }
        } catch (final IOException e) {
            assertNull(e);
            fail("unepected error");
        }
        try {
            final PipedConsoleManager test = new PipedConsoleManager();
            cmd = new ParametrizedCommand(test, "name") {
                {
                    try {
                        addStringParameter(str1, true);
                        addStringParameter(str2, false);
                        addBooleanParameter(bool1);
                        addBooleanParameter(bool2);
                    } catch (final InvalidParameterException e) {
                        // TODO Auto-generated catch block
                        e.printStackTrace();
                    }
                }

                @Override
                protected void doExecute(final CommandParameters parameters) {
                    assertEquals(str2, parameters.get(str1));
                }

                @Override
                public String tip() {
                    return "";
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            };
            test.close();
            cmd.execute("-" + str1, str2);
            cmd.execute("-" + addParam);
            fail("Closed manager shall cause error");
        } catch (final IOException e) {
            assertNull(e);
        } catch (final CommandRunException e) {
            assertNotNull(e);
        }

    }

    /** Test method for
     * {@link fr.bigeon.gclc.command.ParametrizedCommand#ParametrizedCommand(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String)}. */
    @Test
    public final void testParametrizedCommandConsoleManagerString() {
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            final ParametrizedCommand cmd = new ParametrizedCommand(test, "name") {

                @Override
                protected void doExecute(final CommandParameters parameters) {
                    //
                }

                @Override
                public String tip() {
                    return null;
                }
                @Override
                protected String usageDetail() {
                     return null;
                }
            };
            assertTrue(cmd.isStrict());
            assertTrue(cmd.isInteractive());
        } catch (final IOException e) {
            fail("Unexpected exception in creation");
            assertNull(e);
        }
        ParametrizedCommand cmd = new ParametrizedCommand(null, "name") {
            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        assertTrue(cmd.isStrict());
        assertFalse(cmd.isInteractive());
        cmd = new ParametrizedCommand("name") {
            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        assertTrue(cmd.isStrict());
        assertFalse(cmd.isInteractive());
    }

    /** Test method for
     * {@link fr.bigeon.gclc.command.ParametrizedCommand#ParametrizedCommand(fr.bigeon.gclc.manager.ConsoleManager, java.lang.String, boolean)}. */
    @Test
    public final void testParametrizedCommandConsoleManagerStringBoolean() {
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            final ParametrizedCommand cmd = new ParametrizedCommand(test, "name",
                    false) {
                @Override
                protected void doExecute(final CommandParameters parameters) {
                    //
                }

                @Override
                public String tip() {
                    return null;
                }

                @Override
                protected String usageDetail() {
                     return null;
                }
            };
            assertFalse(cmd.isStrict());
            assertTrue(cmd.isInteractive());
        } catch (final IOException e) {
            fail("Unexpected exception in creation");
            assertNull(e);
        }
        ParametrizedCommand cmd = new ParametrizedCommand(null, "name", false) {
            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        assertFalse(cmd.isStrict());
        assertFalse(cmd.isInteractive());
        cmd = new ParametrizedCommand("name", false) {
            @Override
            protected void doExecute(final CommandParameters parameters) {
                //
            }

            @Override
            public String tip() {
                return null;
            }

            @Override
            protected String usageDetail() {
                 return null;
            }
        };
        assertFalse(cmd.isStrict());
        assertFalse(cmd.isInteractive());
    }

}
