/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.proc.TaskPoolTest.java
 * Created on: Aug 19, 2017
 */
package fr.bigeon.gclc.proc;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import org.junit.Test;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class TaskPoolTest {

    /** Test method for
     * {@link fr.bigeon.gclc.proc.TaskPool#add(fr.bigeon.gclc.proc.Task)}. */
    @Test
    public final void testAdd() {
        final TaskPool pool = new TaskPool();
        Task task = null;
        try {
            pool.add(task);
            fail("Expected a null pointer exception");
        } catch (final IllegalArgumentException e) {
            // ok
        }
        task = new Task() {
            private final Object lock = new Object();
            private boolean running;
            private InterruptionListener listener;

            @Override
            public void addInterruptionListener(final InterruptionListener listener) {
                this.listener = listener;
            }

            @Override
            public String getName() {
                return "Test";
            }

            @Override
            public boolean isRunning() {
                return running;
            }

            @Override
            public void rmInterruptionListener(final InterruptionListener listener) {
                //
            }

            @Override
            public void run() {
                synchronized (lock) {
                    while (running) {
                        try {
                            lock.wait(100);
                        } catch (final InterruptedException e) {
                            // TODO Auto-generated catch block
                            e.printStackTrace();
                        }
                    }
                }
                listener.interrupted();
            }

            @Override
            public void setRunning(final boolean running) {
                synchronized (lock) {
                    this.running = running;
                }
                //
            }
        };
        pool.add(task);
        assertEquals(1, pool.getPIDs().size());

        for (final String pid : pool.getPIDs()) {
            assertEquals(task, pool.get(pid));
        }
        final Thread th = new Thread(task);
        th.start();
        task.setRunning(false);
        try {
            th.join(1000);
        } catch (final InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        assertEquals(0, pool.getPIDs().size());
    }
}
